import { TAB_SIZE } from './mod.ts';
import * as Util from './utils.ts';

/**
 * One row of text in the current document
 */
export class Row {
	/**
	 * The actual characters in the current row
	 */
	chars: string[] = [];

	/**
	 * The characters rendered for the current row
	 * (like replacing tabs with spaces)
	 */
	render: string[] = [];

	constructor(s: string = '') {
		this.chars = Util.chars(s);
		this.render = [];
	}

	public get size(): number {
		return this.chars.length;
	}

	public get rsize(): number {
		return this.render.length;
	}

	public rstring(offset: number = 0): string {
		return this.render.slice(offset).join('');
	}

	public insertChar(at: number, c: string): void {
		const newSlice = Util.chars(c);
		if (at >= this.size) {
			this.chars = this.chars.concat(newSlice);
		} else {
			const front = this.chars.slice(0, at + 1);
			const back = this.chars.slice(at + 1);
			this.chars = front.concat(newSlice, back);
		}
	}

	public cxToRx(cx: number): number {
		let rx = 0;
		let j = 0;
		for (; j < cx; j++) {
			if (this.chars[j] === '\t') {
				rx += (TAB_SIZE - 1) - (rx % TAB_SIZE);
			}
			rx++;
		}

		return rx;
	}

	public toString(): string {
		return this.chars.join('');
	}
}

export default Row;
